@@ -6,6 +6,8 @@ mod constants;
 #[cfg(any(target_os = "android", target_os = "linux"))]
 mod copy_file_range;
 #[cfg(not(target_os = "redox"))]
+mod cwd;
+#[cfg(not(target_os = "redox"))]
 mod dir;
 #[cfg(not(any(
     target_os = "ios",
@@ -36,6 +38,8 @@ mod makedev;
 mod memfd_create;
 #[cfg(any(target_os = "android", target_os = "linux"))]
 mod openat2;
+#[cfg(target_os = "linux")]
+mod sendfile;
 #[cfg(all(target_os = "linux", target_env = "gnu"))]
 mod statx;
 
@@ -45,8 +49,7 @@ pub use at::chmodat;
 pub use at::fclonefileat;
 #[cfg(not(target_os = "redox"))]
 pub use at::{
-    accessat, cwd, linkat, mkdirat, openat, readlinkat, renameat, statat, symlinkat, unlinkat,
-    utimensat,
+    accessat, linkat, mkdirat, openat, readlinkat, renameat, statat, symlinkat, unlinkat, utimensat,
 };
 #[cfg(not(target_os = "redox"))]
 pub use constants::AtFlags;
@@ -58,7 +61,9 @@ pub use constants::{CloneFlags, CopyfileFlags};
 #[cfg(any(target_os = "android", target_os = "linux"))]
 pub use copy_file_range::copy_file_range;
 #[cfg(not(target_os = "redox"))]
-pub use dir::{Dir, Entry, SeekLoc};
+pub use cwd::cwd;
+#[cfg(not(target_os = "redox"))]
+pub use dir::{Dir, Entry};
 #[cfg(not(any(
     target_os = "ios",
     target_os = "macos",
@@ -112,6 +117,8 @@ pub use makedev::{major, makedev, minor};
 pub use memfd_create::{memfd_create, MemfdFlags};
 #[cfg(any(target_os = "android", target_os = "linux"))]
 pub use openat2::openat2;
+#[cfg(target_os = "linux")]
+pub use sendfile::sendfile;
 #[cfg(all(target_os = "linux", target_env = "gnu"))]
 pub use statx::statx;
 
@@ -246,7 +253,7 @@ pub const PROC_SUPER_MAGIC: FsWord = 0x0000_9fa0;
 /// The filesystem magic number for procfs.
 /// <https://man7.org/linux/man-pages/man2/fstatfs.2.html#DESCRIPTION>
 #[cfg(linux_raw)]
-pub const PROC_SUPER_MAGIC: FsWord = linux_raw_sys::general::PROC_SUPER_MAGIC;
+pub const PROC_SUPER_MAGIC: FsWord = linux_raw_sys::general::PROC_SUPER_MAGIC as FsWord;
 
 /// Re-export `mode_t`.
 #[cfg(libc)]
