@@ -5,19 +5,39 @@ use crate::io;
 use crate::negone_err;
 #[cfg(all(libc, not(target_os = "redox")))]
 use crate::zero_ok;
-use io_lifetimes::{AsFd, BorrowedFd, OwnedFd};
+use bitflags::bitflags;
+use io_lifetimes::{AsFd, BorrowedFd, IntoFd, OwnedFd};
 #[cfg(all(libc, not(any(target_os = "wasi", target_os = "fuchsia"))))]
 use std::ffi::OsString;
-#[cfg(not(target_os = "redox"))]
+#[cfg(all(libc, not(target_os = "redox")))]
 use std::mem::MaybeUninit;
 #[cfg(all(libc, unix, not(any(target_os = "wasi", target_os = "fuchsia"))))]
 use std::os::unix::ffi::OsStringExt;
 #[cfg(libc)]
 use {
     errno::errno,
-    unsafe_io::os::posish::{AsRawFd, FromRawFd},
+    unsafe_io::os::posish::{AsRawFd, FromRawFd, IntoRawFd},
 };
 
+#[cfg(libc)]
+bitflags! {
+    /// `O_*` constants for use with `dup2`.
+    pub struct DupFlags: std::os::raw::c_int {
+        /// `O_CLOEXEC`
+        #[cfg(not(any(target_os = "android", target_os = "macos", target_os = "ios", target_os = "redox")))] // Android 5.0 has dup3, but libc doesn't have bindings
+        const CLOEXEC = libc::O_CLOEXEC;
+    }
+}
+
+#[cfg(linux_raw)]
+bitflags! {
+    /// `O_*` constants for use with `dup2`.
+    pub struct DupFlags: std::os::raw::c_uint {
+        /// `O_CLOEXEC`
+        const CLOEXEC = linux_raw_sys::general::O_CLOEXEC;
+    }
+}
+
 /// `ioctl(fd, FIONREAD)`.
 #[cfg(not(target_os = "redox"))]
 #[inline]
@@ -164,11 +184,13 @@ fn _is_read_write(fd: BorrowedFd<'_>) -> io::Result<(bool, bool)> {
         // Do a `recv` with `PEEK` and `DONTWAIT` for 1 byte. A 0 indicates
         // the read side is shut down; an `EWOULDBLOCK` indicates the read
         // side is still open.
-        let mut buf = MaybeUninit::<u8>::uninit();
-        let mut view = unsafe { std::slice::from_raw_parts_mut(buf.as_mut_ptr(), 1) };
+        //
+        // TODO: This code would benefit from having a better way to read into
+        // uninitialized memory.
+        let mut buf = [0];
         match crate::linux_raw::recv(
             fd,
-            &mut view,
+            &mut buf,
             linux_raw_sys::general::MSG_PEEK | linux_raw_sys::general::MSG_DONTWAIT,
         ) {
             Ok(0) => read = false,
@@ -203,6 +225,11 @@ fn _is_read_write(fd: BorrowedFd<'_>) -> io::Result<(bool, bool)> {
 }
 
 /// `dup(fd)`
+///
+/// Note that this does not set the `O_CLOEXEC` flag. To do a dup that does
+/// set `O_CLOEXEC, use [`fcntl_dupfd_cloexec`].
+///
+/// [`fcntl_dupfd_cloexec`]: posish::fs::fcntl_dupfd_cloexec
 #[cfg(not(target_os = "wasi"))]
 #[inline]
 pub fn dup<'f, Fd: AsFd<'f>>(fd: Fd) -> io::Result<OwnedFd> {
@@ -223,6 +250,60 @@ fn _dup(fd: BorrowedFd<'_>) -> io::Result<OwnedFd> {
     crate::linux_raw::dup(fd)
 }
 
+/// `dup3(fd, new, flags)`
+#[cfg(not(target_os = "wasi"))]
+#[inline]
+#[doc(alias = "dup3")]
+pub fn dup2<'f, Fd: AsFd<'f>, NewFd: IntoFd>(
+    fd: Fd,
+    new: NewFd,
+    flags: DupFlags,
+) -> io::Result<OwnedFd> {
+    let fd = fd.as_fd();
+    let new = new.into_fd();
+    _dup2(fd, new, flags)
+}
+
+#[cfg(all(
+    libc,
+    not(any(
+        target_os = "android",
+        target_os = "macos",
+        target_os = "ios",
+        target_os = "redox",
+        target_os = "wasi"
+    ))
+))]
+fn _dup2(fd: BorrowedFd<'_>, new: OwnedFd, flags: DupFlags) -> io::Result<OwnedFd> {
+    unsafe {
+        let new_fd = negone_err(libc::dup3(fd.as_raw_fd(), new.into_raw_fd(), flags.bits()))?;
+        Ok(OwnedFd::from_raw_fd(new_fd))
+    }
+}
+
+#[cfg(all(
+    libc,
+    any(
+        target_os = "android",
+        target_os = "macos",
+        target_os = "ios",
+        target_os = "redox"
+    )
+))]
+fn _dup2(fd: BorrowedFd<'_>, new: OwnedFd, _flags: DupFlags) -> io::Result<OwnedFd> {
+    unsafe {
+        // Android 5.0 has dup3, but libc doesn't have bindings
+        let new_fd = negone_err(libc::dup2(fd.as_raw_fd(), new.into_raw_fd()))?;
+        Ok(OwnedFd::from_raw_fd(new_fd))
+    }
+}
+
+#[cfg(linux_raw)]
+#[inline]
+fn _dup2(fd: BorrowedFd<'_>, new: OwnedFd, flags: DupFlags) -> io::Result<OwnedFd> {
+    crate::linux_raw::dup3(fd, new, flags.bits())
+}
+
 /// `ttyname_r(fd)`
 ///
 /// If `reuse` is non-empty, reuse its buffer to store the result if possible.
@@ -235,29 +316,27 @@ pub fn ttyname<'f, Fd: AsFd<'f>>(dirfd: Fd, reuse: OsString) -> io::Result<OsStr
 
 #[cfg(all(libc, not(any(target_os = "wasi", target_os = "fuchsia"))))]
 fn _ttyname(dirfd: BorrowedFd<'_>, reuse: OsString) -> io::Result<OsString> {
+    // This code would benefit from having a better way to read into
+    // uninitialized memory, but that requires `unsafe`.
     let mut buffer = reuse.into_vec();
-
-    // Start with a buffer big enough for the vast majority of paths.
-    // This and the `reserve` below would be a good candidate for `try_reserve`.
-    // https://github.com/rust-lang/rust/issues/48043
     buffer.clear();
-    buffer.reserve(256);
+    buffer.resize(256, 0u8);
 
-    unsafe {
-        loop {
-            match libc::ttyname_r(
+    loop {
+        match unsafe {
+            libc::ttyname_r(
                 dirfd.as_raw_fd() as libc::c_int,
                 buffer.as_mut_ptr().cast::<libc::c_char>(),
-                buffer.capacity(),
-            ) {
-                // Use `Vec`'s builtin capacity-doubling strategy.
-                libc::ERANGE => buffer.reserve(1),
-                0 => {
-                    buffer.set_len(libc::strlen(buffer.as_ptr().cast::<libc::c_char>()));
-                    return Ok(OsString::from_vec(buffer));
-                }
-                errno => return Err(io::Error(errno)),
+                buffer.len(),
+            )
+        } {
+            libc::ERANGE => buffer.resize(buffer.len() * 2, 0u8),
+            0 => {
+                let len = buffer.iter().position(|x| *x == b'\0').unwrap();
+                buffer.resize(len, 0u8);
+                return Ok(OsString::from_vec(buffer));
             }
+            errno => return Err(io::Error(errno)),
         }
     }
 }
