@@ -5,15 +5,16 @@ mod fd;
 mod ioctl;
 mod pipe;
 mod poll;
+#[cfg(libc)]
+mod poll_fd;
 #[cfg(any(target_os = "macos", target_os = "ios"))]
 mod rdadvise;
 mod read_write;
+mod stdio;
 #[cfg(any(target_os = "android", target_os = "linux"))]
 mod userfaultfd;
 
 pub use error::{Error, Result};
-#[cfg(not(target_os = "wasi"))]
-pub use fd::dup;
 #[cfg(not(target_os = "redox"))]
 pub use fd::ioctl_fionread;
 #[cfg(not(target_os = "redox"))]
@@ -21,6 +22,8 @@ pub use fd::is_read_write;
 pub use fd::isatty;
 #[cfg(all(libc, not(any(target_os = "wasi", target_os = "fuchsia"))))]
 pub use fd::ttyname;
+#[cfg(not(target_os = "wasi"))]
+pub use fd::{dup, dup2, DupFlags};
 #[cfg(any(target_os = "ios", target_os = "macos"))]
 pub use ioctl::ioctl_fioclex;
 #[cfg(not(target_os = "wasi"))]
@@ -37,6 +40,7 @@ pub use read_write::{pread, pwrite, read, readv, write, writev};
 pub use read_write::{preadv, pwritev};
 #[cfg(any(linux_raw, all(libc, target_os = "linux", target_env = "gnu")))]
 pub use read_write::{preadv2, pwritev2};
+pub use stdio::{stderr, stdin, stdout};
 #[cfg(any(target_os = "android", target_os = "linux"))]
 pub use userfaultfd::{userfaultfd, UserFaultFdFlags};
 
