@@ -1,5 +1,5 @@
 use crate::{
-    as_ptr, io,
+    io,
     net::{SocketAddr, SocketAddrUnix, SocketAddrV4, SocketAddrV6},
 };
 use bitflags::bitflags;
@@ -7,77 +7,82 @@ use io_lifetimes::{AsFd, BorrowedFd, OwnedFd};
 use std::os::raw::c_int;
 #[cfg(linux_raw)]
 use std::os::raw::c_uint;
-use std::{
-    mem::{size_of, MaybeUninit},
-    ptr,
-};
 #[cfg(libc)]
 use {
-    crate::{negone_err, zero_ok},
+    super::sockaddr_header::decode_sockaddr,
+    crate::{as_ptr, negone_err, zero_ok},
     libc::{sockaddr_storage, socklen_t},
+    std::mem::{size_of, MaybeUninit},
     unsafe_io::os::posish::{AsRawFd, FromRawFd},
 };
 
 /// `SOCK_*` constants for `socket`.
 #[cfg(libc)]
 #[derive(Debug, Clone, Copy, Eq, PartialEq, Hash)]
-#[repr(u32)]
-#[non_exhaustive]
-pub enum SocketType {
+#[repr(transparent)]
+pub struct SocketType(pub(crate) u32);
+
+#[cfg(cfg)]
+impl SocketType {
     /// `SOCK_STREAM`.
-    Stream = libc::SOCK_STREAM as u32,
+    pub const STREAM: Self = Self(libc::SOCK_STREAM as u32);
 
     /// `SOCK_DGRAM`.
-    Datagram = libc::SOCK_DGRAM as u32,
+    pub const DGRAM: Self = Self(libc::SOCK_DGRAM as u32);
 
     /// `SOCK_SEQPACKET`.
-    SeqPacket = libc::SOCK_SEQPACKET as u32,
+    pub const SEQPACKET: Self = Self(libc::SOCK_SEQPACKET as u32);
 
     /// `SOCK_RAW`.
-    Raw = libc::SOCK_RAW as u32,
+    pub const RAW: Self = Self(libc::SOCK_RAW as u32);
 
     /// `SOCK_RDM`.
-    Rdm = libc::SOCK_RDM as u32,
+    pub const RDM: Self = Self(libc::SOCK_RDM as u32);
 }
 
 /// `SOCK_*` constants for `socket`.
 #[cfg(linux_raw)]
 #[derive(Debug, Clone, Copy, Eq, PartialEq, Hash)]
-#[repr(u32)]
-#[non_exhaustive]
-pub enum SocketType {
+#[repr(transparent)]
+pub struct SocketType(pub(crate) u32);
+
+#[cfg(linux_raw)]
+impl SocketType {
     /// `SOCK_STREAM`.
-    Stream = linux_raw_sys::general::SOCK_STREAM,
+    pub const STREAM: Self = Self(linux_raw_sys::general::SOCK_STREAM);
 
     /// `SOCK_DGRAM`.
-    Datagram = linux_raw_sys::general::SOCK_DGRAM,
+    pub const DGRAM: Self = Self(linux_raw_sys::general::SOCK_DGRAM);
 
     /// `SOCK_SEQPACKET`.
-    SeqPacket = linux_raw_sys::general::SOCK_SEQPACKET,
+    pub const SEQPACKET: Self = Self(linux_raw_sys::general::SOCK_SEQPACKET);
 
     /// `SOCK_RAW`.
-    Raw = linux_raw_sys::general::SOCK_RAW,
+    pub const RAW: Self = Self(linux_raw_sys::general::SOCK_RAW);
 
     /// `SOCK_RDM`.
-    Rdm = linux_raw_sys::general::SOCK_RDM,
+    pub const RDM: Self = Self(linux_raw_sys::general::SOCK_RDM);
 }
 
 /// `AF_*` constants.
 #[cfg(libc)]
 #[derive(Debug, Clone, Copy, Eq, PartialEq, Hash)]
-#[repr(u32)]
-#[non_exhaustive]
-pub enum AddressFamily {
-    /// `AF_LOCAL`, aka `AF_UNIX`
-    #[doc(alias = "Unix")]
-    Local = libc::AF_LOCAL as u32,
+#[repr(transparent)]
+pub struct AddressFamily(pub(crate) libc::sa_family_t);
 
+/// `AF_*` constants.
+#[cfg(linux_raw)]
+#[derive(Debug, Clone, Copy, Eq, PartialEq, Hash)]
+#[repr(transparent)]
+// Older Linux versions didn't export this typedef.
+pub struct AddressFamily(pub(crate) linux_raw_sys::v5_4::general::__kernel_sa_family_t);
+
+#[cfg(libc)]
+impl AddressFamily {
     /// `AF_INET`
-    Inet = libc::AF_INET as u32,
-
+    pub const INET: Self = Self(libc::AF_INET as _);
     /// `AF_INET6`
-    Inet6 = libc::AF_INET6 as u32,
-
+    pub const INET6: Self = Self(libc::AF_INET6 as _);
     /// `AF_NETLINK`
     #[cfg(not(any(
         target_os = "freebsd",
@@ -85,27 +90,23 @@ pub enum AddressFamily {
         target_os = "macos",
         target_os = "netbsd"
     )))]
-    Netlink = libc::AF_NETLINK as u32,
+    pub const NETLINK: Self = Self(libc::AF_NETLINK as _);
+    /// `AF_UNIX`, aka `AF_LOCAL`
+    #[doc(alias = "Local")]
+    pub const UNIX: Self = Self(libc::AF_UNIX as _);
 }
 
-/// `AF_*` constants.
 #[cfg(linux_raw)]
-#[derive(Debug, Clone, Copy, Eq, PartialEq, Hash)]
-#[repr(u32)]
-#[non_exhaustive]
-pub enum AddressFamily {
-    /// `AF_LOCAL`, aka `AF_UNIX`
-    #[doc(alias = "Unix")]
-    Local = linux_raw_sys::general::AF_LOCAL,
-
+impl AddressFamily {
     /// `AF_INET`
-    Inet = linux_raw_sys::general::AF_INET,
-
+    pub const INET: Self = Self(linux_raw_sys::general::AF_INET as _);
     /// `AF_INET6`
-    Inet6 = linux_raw_sys::general::AF_INET6,
-
+    pub const INET6: Self = Self(linux_raw_sys::general::AF_INET6 as _);
     /// `AF_NETLINK`
-    Netlink = linux_raw_sys::general::AF_NETLINK,
+    pub const NETLINK: Self = Self(linux_raw_sys::general::AF_NETLINK as _);
+    /// `AF_UNIX`, aka `AF_LOCAL`
+    #[doc(alias = "Local")]
+    pub const UNIX: Self = Self(linux_raw_sys::general::AF_LOCAL as _);
 }
 
 /// `IPPROTO_*`
@@ -316,8 +317,8 @@ pub fn socket(domain: AddressFamily, type_: SocketType, protocol: Protocol) -> i
 fn _socket(domain: AddressFamily, type_: SocketType, protocol: Protocol) -> io::Result<OwnedFd> {
     unsafe {
         let raw_fd = negone_err(libc::socket(
-            domain as c_int,
-            type_ as c_int,
+            domain.0 as c_int,
+            type_.0 as c_int,
             protocol as c_int,
         ))?;
         Ok(OwnedFd::from_raw_fd(raw_fd))
@@ -326,41 +327,19 @@ fn _socket(domain: AddressFamily, type_: SocketType, protocol: Protocol) -> io::
 
 #[cfg(linux_raw)]
 fn _socket(domain: AddressFamily, type_: SocketType, protocol: Protocol) -> io::Result<OwnedFd> {
-    crate::linux_raw::socket(domain as c_uint, type_ as c_uint, protocol as c_uint)
-}
-
-/// `bind(sockfd, addr, sizeof(struct sockaddr_un))`
-#[inline]
-pub fn bind_un<'f, Fd: AsFd<'f>>(sockfd: Fd, addr: &SocketAddrUnix) -> io::Result<()> {
-    let sockfd = sockfd.as_fd();
-    _bind_un(sockfd, addr)
-}
-
-#[cfg(libc)]
-fn _bind_un(sockfd: BorrowedFd<'_>, addr: &SocketAddrUnix) -> io::Result<()> {
-    unsafe {
-        zero_ok(libc::bind(
-            sockfd.as_raw_fd(),
-            as_ptr(addr).cast::<_>(),
-            size_of::<SocketAddrUnix>() as socklen_t,
-        ))
-    }
-}
-
-#[cfg(linux_raw)]
-fn _bind_un(sockfd: BorrowedFd<'_>, addr: &SocketAddrUnix) -> io::Result<()> {
-    crate::linux_raw::bind_un(sockfd, addr)
+    crate::linux_raw::socket(domain.0.into(), type_.0, protocol as c_uint)
 }
 
 /// `bind(sockfd, addr, sizeof(struct sockaddr_in))`
 #[inline]
-pub fn bind_in<'f, Fd: AsFd<'f>>(sockfd: Fd, addr: &SocketAddrV4) -> io::Result<()> {
+#[doc(alias("bind"))]
+pub fn bind_v4<'f, Fd: AsFd<'f>>(sockfd: Fd, addr: &SocketAddrV4) -> io::Result<()> {
     let sockfd = sockfd.as_fd();
-    _bind_in(sockfd, addr)
+    _bind_v4(sockfd, addr)
 }
 
 #[cfg(libc)]
-fn _bind_in(sockfd: BorrowedFd<'_>, addr: &SocketAddrV4) -> io::Result<()> {
+fn _bind_v4(sockfd: BorrowedFd<'_>, addr: &SocketAddrV4) -> io::Result<()> {
     unsafe {
         zero_ok(libc::bind(
             sockfd.as_raw_fd(),
@@ -371,19 +350,20 @@ fn _bind_in(sockfd: BorrowedFd<'_>, addr: &SocketAddrV4) -> io::Result<()> {
 }
 
 #[cfg(linux_raw)]
-fn _bind_in(sockfd: BorrowedFd<'_>, addr: &SocketAddrV4) -> io::Result<()> {
+fn _bind_v4(sockfd: BorrowedFd<'_>, addr: &SocketAddrV4) -> io::Result<()> {
     crate::linux_raw::bind_in(sockfd, addr)
 }
 
 /// `bind(sockfd, addr, sizeof(struct sockaddr_in6))`
 #[inline]
-pub fn bind_in6<'f, Fd: AsFd<'f>>(sockfd: Fd, addr: &SocketAddrV6) -> io::Result<()> {
+#[doc(alias("bind"))]
+pub fn bind_v6<'f, Fd: AsFd<'f>>(sockfd: Fd, addr: &SocketAddrV6) -> io::Result<()> {
     let sockfd = sockfd.as_fd();
-    _bind_in6(sockfd, addr)
+    _bind_v6(sockfd, addr)
 }
 
 #[cfg(libc)]
-fn _bind_in6(sockfd: BorrowedFd<'_>, addr: &SocketAddrV6) -> io::Result<()> {
+fn _bind_v6(sockfd: BorrowedFd<'_>, addr: &SocketAddrV6) -> io::Result<()> {
     unsafe {
         zero_ok(libc::bind(
             sockfd.as_raw_fd(),
@@ -394,21 +374,22 @@ fn _bind_in6(sockfd: BorrowedFd<'_>, addr: &SocketAddrV6) -> io::Result<()> {
 }
 
 #[cfg(linux_raw)]
-fn _bind_in6(sockfd: BorrowedFd<'_>, addr: &SocketAddrV6) -> io::Result<()> {
+fn _bind_v6(sockfd: BorrowedFd<'_>, addr: &SocketAddrV6) -> io::Result<()> {
     crate::linux_raw::bind_in6(sockfd, addr)
 }
 
-/// `connect(sockfd, addr, sizeof(struct sockaddr_un))`
+/// `bind(sockfd, addr, sizeof(struct sockaddr_un))`
 #[inline]
-pub fn connect_un<'f, Fd: AsFd<'f>>(sockfd: Fd, addr: &SocketAddrUnix) -> io::Result<()> {
+#[doc(alias("bind"))]
+pub fn bind_unix<'f, Fd: AsFd<'f>>(sockfd: Fd, addr: &SocketAddrUnix) -> io::Result<()> {
     let sockfd = sockfd.as_fd();
-    _connect_un(sockfd, addr)
+    _bind_unix(sockfd, addr)
 }
 
 #[cfg(libc)]
-fn _connect_un(sockfd: BorrowedFd<'_>, addr: &SocketAddrUnix) -> io::Result<()> {
+fn _bind_unix(sockfd: BorrowedFd<'_>, addr: &SocketAddrUnix) -> io::Result<()> {
     unsafe {
-        zero_ok(libc::connect(
+        zero_ok(libc::bind(
             sockfd.as_raw_fd(),
             as_ptr(addr).cast::<_>(),
             size_of::<SocketAddrUnix>() as socklen_t,
@@ -417,19 +398,20 @@ fn _connect_un(sockfd: BorrowedFd<'_>, addr: &SocketAddrUnix) -> io::Result<()>
 }
 
 #[cfg(linux_raw)]
-fn _connect_un(sockfd: BorrowedFd<'_>, addr: &SocketAddrUnix) -> io::Result<()> {
-    crate::linux_raw::connect_un(sockfd, addr)
+fn _bind_unix(sockfd: BorrowedFd<'_>, addr: &SocketAddrUnix) -> io::Result<()> {
+    crate::linux_raw::bind_un(sockfd, addr)
 }
 
 /// `connect(sockfd, addr, sizeof(struct sockaddr_in))`
 #[inline]
-pub fn connect_in<'f, Fd: AsFd<'f>>(sockfd: Fd, addr: &SocketAddrV4) -> io::Result<()> {
+#[doc(alias("connect"))]
+pub fn connect_v4<'f, Fd: AsFd<'f>>(sockfd: Fd, addr: &SocketAddrV4) -> io::Result<()> {
     let sockfd = sockfd.as_fd();
-    _connect_in(sockfd, addr)
+    _connect_v4(sockfd, addr)
 }
 
 #[cfg(libc)]
-fn _connect_in(sockfd: BorrowedFd<'_>, addr: &SocketAddrV4) -> io::Result<()> {
+fn _connect_v4(sockfd: BorrowedFd<'_>, addr: &SocketAddrV4) -> io::Result<()> {
     unsafe {
         zero_ok(libc::connect(
             sockfd.as_raw_fd(),
@@ -440,19 +422,20 @@ fn _connect_in(sockfd: BorrowedFd<'_>, addr: &SocketAddrV4) -> io::Result<()> {
 }
 
 #[cfg(linux_raw)]
-fn _connect_in(sockfd: BorrowedFd<'_>, addr: &SocketAddrV4) -> io::Result<()> {
+fn _connect_v4(sockfd: BorrowedFd<'_>, addr: &SocketAddrV4) -> io::Result<()> {
     crate::linux_raw::connect_in(sockfd, addr)
 }
 
 /// `connect(sockfd, addr, sizeof(struct sockaddr_in6))`
 #[inline]
-pub fn connect_in6<'f, Fd: AsFd<'f>>(sockfd: Fd, addr: &SocketAddrV6) -> io::Result<()> {
+#[doc(alias("connect"))]
+pub fn connect_v6<'f, Fd: AsFd<'f>>(sockfd: Fd, addr: &SocketAddrV6) -> io::Result<()> {
     let sockfd = sockfd.as_fd();
-    _connect_in6(sockfd, addr)
+    _connect_v6(sockfd, addr)
 }
 
 #[cfg(libc)]
-fn _connect_in6(sockfd: BorrowedFd<'_>, addr: &SocketAddrV6) -> io::Result<()> {
+fn _connect_v6(sockfd: BorrowedFd<'_>, addr: &SocketAddrV6) -> io::Result<()> {
     unsafe {
         zero_ok(libc::connect(
             sockfd.as_raw_fd(),
@@ -463,10 +446,34 @@ fn _connect_in6(sockfd: BorrowedFd<'_>, addr: &SocketAddrV6) -> io::Result<()> {
 }
 
 #[cfg(linux_raw)]
-fn _connect_in6(sockfd: BorrowedFd<'_>, addr: &SocketAddrV6) -> io::Result<()> {
+fn _connect_v6(sockfd: BorrowedFd<'_>, addr: &SocketAddrV6) -> io::Result<()> {
     crate::linux_raw::connect_in6(sockfd, addr)
 }
 
+/// `connect(sockfd, addr, sizeof(struct sockaddr_un))`
+#[doc(alias("connect"))]
+#[inline]
+pub fn connect_unix<'f, Fd: AsFd<'f>>(sockfd: Fd, addr: &SocketAddrUnix) -> io::Result<()> {
+    let sockfd = sockfd.as_fd();
+    _connect_unix(sockfd, addr)
+}
+
+#[cfg(libc)]
+fn _connect_unix(sockfd: BorrowedFd<'_>, addr: &SocketAddrUnix) -> io::Result<()> {
+    unsafe {
+        zero_ok(libc::connect(
+            sockfd.as_raw_fd(),
+            as_ptr(addr).cast::<_>(),
+            size_of::<SocketAddrUnix>() as socklen_t,
+        ))
+    }
+}
+
+#[cfg(linux_raw)]
+fn _connect_unix(sockfd: BorrowedFd<'_>, addr: &SocketAddrUnix) -> io::Result<()> {
+    crate::linux_raw::connect_un(sockfd, addr)
+}
+
 /// `listen(fd, backlog)`
 #[inline]
 pub fn listen<'f, Fd: AsFd<'f>>(sockfd: Fd, backlog: c_int) -> io::Result<()> {
@@ -508,23 +515,7 @@ fn _accept(sockfd: BorrowedFd<'_>, flags: AcceptFlags) -> io::Result<(OwnedFd, S
             flags.bits(),
         ))?;
         let owned_fd = OwnedFd::from_raw_fd(raw_fd);
-        let storage = storage.assume_init();
-        let addr = match i32::from(storage.ss_family) {
-            libc::AF_INET => {
-                assert!(len as usize >= size_of::<SocketAddrV4>());
-                SocketAddr::V4(ptr::read(as_ptr(&storage).cast::<_>()))
-            }
-            libc::AF_INET6 => {
-                assert!(len as usize >= size_of::<SocketAddrV6>());
-                SocketAddr::V6(ptr::read(as_ptr(&storage).cast::<_>()))
-            }
-            libc::AF_LOCAL => {
-                assert!(len as usize >= size_of::<SocketAddrUnix>());
-                SocketAddr::Unix(ptr::read(as_ptr(&storage).cast::<_>()))
-            }
-            _ => panic!(),
-        };
-        Ok((owned_fd, addr))
+        Ok((owned_fd, decode_sockaddr(storage.as_ptr())))
     }
 }
 
@@ -541,48 +532,14 @@ fn _accept(sockfd: BorrowedFd<'_>, _flags: AcceptFlags) -> io::Result<(OwnedFd,
             &mut len,
         ))?;
         let owned_fd = OwnedFd::from_raw_fd(raw_fd);
-        let storage = storage.assume_init();
-        let addr = match i32::from(storage.ss_family) {
-            libc::AF_INET => {
-                assert!(len as usize >= size_of::<SocketAddrV4>());
-                SocketAddr::V4(ptr::read(as_ptr(&storage).cast::<_>()))
-            }
-            libc::AF_INET6 => {
-                assert!(len as usize >= size_of::<SocketAddrV6>());
-                SocketAddr::V6(ptr::read(as_ptr(&storage).cast::<_>()))
-            }
-            libc::AF_LOCAL => {
-                assert!(len as usize >= size_of::<SocketAddrUnix>());
-                SocketAddr::Unix(ptr::read(as_ptr(&storage).cast::<_>()))
-            }
-            _ => panic!(),
-        };
-        Ok((owned_fd, addr))
+        Ok((owned_fd, decode_sockaddr(storage.as_ptr())))
     }
 }
 
 #[cfg(linux_raw)]
 #[inline]
 fn _accept(sockfd: BorrowedFd<'_>, flags: AcceptFlags) -> io::Result<(OwnedFd, SocketAddr)> {
-    let (owned_fd, storage, len) = crate::linux_raw::accept(sockfd, flags.bits())?;
-    let addr = unsafe {
-        match u32::from(storage.ss_family) {
-            linux_raw_sys::general::AF_INET => {
-                assert!(len as usize >= size_of::<SocketAddrV4>());
-                SocketAddr::V4(ptr::read(as_ptr(&storage).cast::<_>()))
-            }
-            linux_raw_sys::general::AF_INET6 => {
-                assert!(len as usize >= size_of::<SocketAddrV6>());
-                SocketAddr::V6(ptr::read(as_ptr(&storage).cast::<_>()))
-            }
-            linux_raw_sys::general::AF_LOCAL => {
-                assert!(len as usize >= size_of::<SocketAddrUnix>());
-                SocketAddr::Unix(ptr::read(as_ptr(&storage).cast::<_>()))
-            }
-            _ => panic!(),
-        }
-    };
-    Ok((owned_fd, addr))
+    crate::linux_raw::accept(sockfd, flags.bits())
 }
 
 /// `shutdown(fd, how)`
@@ -605,13 +562,13 @@ fn _shutdown(sockfd: BorrowedFd<'_>, how: Shutdown) -> io::Result<()> {
 
 /// `getsockopt(fd, SOL_SOCKET, SO_TYPE)`
 #[inline]
-pub fn socket_type<'f, Fd: AsFd<'f>>(fd: Fd) -> io::Result<SocketType> {
+pub fn getsockopt_socket_type<'f, Fd: AsFd<'f>>(fd: Fd) -> io::Result<SocketType> {
     let fd = fd.as_fd();
-    _socket_type(fd)
+    _getsockopt_socket_type(fd)
 }
 
 #[cfg(libc)]
-fn _socket_type(fd: BorrowedFd<'_>) -> io::Result<SocketType> {
+fn _getsockopt_socket_type(fd: BorrowedFd<'_>) -> io::Result<SocketType> {
     let mut buffer = MaybeUninit::<SocketType>::uninit();
     let mut out_len = size_of::<SocketType>() as socklen_t;
     unsafe {
@@ -632,26 +589,60 @@ fn _socket_type(fd: BorrowedFd<'_>) -> io::Result<SocketType> {
 }
 
 #[cfg(linux_raw)]
-fn _socket_type(fd: BorrowedFd<'_>) -> io::Result<SocketType> {
+fn _getsockopt_socket_type(fd: BorrowedFd<'_>) -> io::Result<SocketType> {
+    crate::linux_raw::getsockopt_socket_type(fd).map(SocketType)
+}
+
+/// `getsockname(fd, addr, len)`
+#[inline]
+pub fn getsockname<'f, Fd: AsFd<'f>>(sockfd: Fd) -> io::Result<SocketAddr> {
+    let sockfd = sockfd.as_fd();
+    _getsockname(sockfd)
+}
+
+#[cfg(libc)]
+fn _getsockname(sockfd: BorrowedFd<'_>) -> io::Result<SocketAddr> {
     unsafe {
-        let mut buffer = MaybeUninit::<SocketType>::uninit();
-        let mut out_len = size_of::<SocketType>() as linux_raw_sys::general::socklen_t;
-        let slice = std::slice::from_raw_parts_mut(
-            buffer.as_mut_ptr().cast::<u8>(),
-            size_of::<SocketType>(),
-        );
-        crate::linux_raw::getsockopt(
-            fd,
-            linux_raw_sys::general::SOL_SOCKET as i32,
-            linux_raw_sys::general::SO_TYPE as i32,
-            slice,
-            &mut out_len,
-        )?;
-        assert_eq!(
-            out_len as usize,
-            size_of::<SocketType>(),
-            "unexpected SocketType size"
-        );
-        Ok(buffer.assume_init())
+        let mut storage = MaybeUninit::<sockaddr_storage>::uninit();
+        let mut len = size_of::<sockaddr_storage>() as socklen_t;
+        zero_ok(libc::getsockname(
+            sockfd.as_raw_fd(),
+            storage.as_mut_ptr().cast::<_>(),
+            &mut len,
+        ))?;
+        Ok(decode_sockaddr(storage.as_ptr()))
     }
 }
+
+#[cfg(linux_raw)]
+#[inline]
+fn _getsockname(sockfd: BorrowedFd<'_>) -> io::Result<SocketAddr> {
+    crate::linux_raw::getsockname(sockfd)
+}
+
+/// `getpeername(fd, addr, len)`
+#[inline]
+pub fn getpeername<'f, Fd: AsFd<'f>>(sockfd: Fd) -> io::Result<SocketAddr> {
+    let sockfd = sockfd.as_fd();
+    _getpeername(sockfd)
+}
+
+#[cfg(libc)]
+fn _getpeername(sockfd: BorrowedFd<'_>) -> io::Result<SocketAddr> {
+    unsafe {
+        let mut storage = MaybeUninit::<sockaddr_storage>::uninit();
+        let mut len = size_of::<sockaddr_storage>() as socklen_t;
+        zero_ok(libc::getpeername(
+            sockfd.as_raw_fd(),
+            storage.as_mut_ptr().cast::<_>(),
+            &mut len,
+        ))?;
+        Ok(decode_sockaddr(storage.as_ptr()))
+    }
+}
+
+#[cfg(linux_raw)]
+#[inline]
+fn _getpeername(sockfd: BorrowedFd<'_>) -> io::Result<SocketAddr> {
+    crate::linux_raw::getpeername(sockfd)
+}
